#include <stdlib.h>
#include <math.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <gtkgl/gdkgl.h>
#include <gtkgl/gtkglarea.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glut.h>

#include "config.h"
#include "wired.h"
#include "gldisplay.h"
#include "node.h"
#include "support.h"


extern GtkWidget* GGLArea;
extern GTree* GNodeTree;
extern GtkWidget* gMenu;

GLuint GSphere;

/* radius of the whole thing */
float gRotationRadius = 2.0;

float gRotateX = 2.0;
float gRotateX2;
float gRotateY = 0.0;
float gRotateY2 = 0.0;

/* for dragging */
int gMouseDownX;
int gMouseDownY;

/* global flags */
gint gMouseMotion;
gint gShowNames = 1;
gint gPaused = 0;

/* camera position--not implemented */
GLfloat gCameraX = 0.0;
GLfloat gCameraY = 3.0;
GLfloat gCameraZ = 3.0;


extern GTree* gNetObjectTree;


void GLShowAddresses( GtkWidget* w )
{	gShowNames = -gShowNames;
}


void GLDisplayPause( GtkWidget* w )
{	if( gPaused )
	{	gPaused = 0;
		gRotateX2 = gRotateX * 15.0;
		gRotateY2 = gRotateY * 15.0;
		gRotateX = 2.0;
		gRotateY = 0.0;
		NetObjectInitializeAll();
	}
	else
	{	gPaused = 1;
		gRotateX = gRotateX2/15.0;
		gRotateY = gRotateY2/15.0;
	}
}


void GLAreaInit( GtkWidget* wi, gpointer unused )
{	if( gtk_gl_area_make_current( GTK_GL_AREA( wi ) ) )
	{	GLfloat position[] = { 1.0, 0.0, 0.0, 0.0 };
		GLfloat lighting[] = { 3.0, 3.0, 3.0, 1.0 };

		int squareSize = (wi->allocation.width >wi->allocation.height) ?
			wi->allocation.height : wi->allocation.width;
		glViewport( 0, 0, squareSize, squareSize );
		glLightfv( GL_LIGHT0, GL_POSITION, position );
		glLightfv( GL_LIGHT0, GL_AMBIENT, lighting );

		glEnable( GL_NORMALIZE );
		glEnable( GL_DEPTH_TEST );
		glEnable( GL_LIGHTING );
		glEnable( GL_LIGHT0 );
		glEnable( GL_BLEND );
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		gluPerspective(  80.0, 1.0, 1.0, 10.0 );
		glMatrixMode( GL_MODELVIEW );
		gluLookAt( 0.0, 2.8, 2.8,
			   0.0, 0.0, 0.0,
			   0.0, 1.0, 0.0 );
		glPushMatrix;

		GSphere = glGenLists(1);
		glNewList( GSphere, GL_COMPILE );
		glutSolidSphere( 0.1, 16, 16 );
		glEndList();

		gtk_gl_area_swapbuffers( GTK_GL_AREA( wi ) );
	}
}


int GLAreaInitResize( GtkWidget* w )
{	GLAreaResize( w, 0 );
	return 0;
}


void GLAreaResize( GtkWidget* w, gpointer unused )
{	int squareSize;

	if( ! GTK_WIDGET_REALIZED( w ) )
	{	gtk_widget_queue_resize( w );
		return;
	}
	if( ! gtk_gl_area_make_current( GTK_GL_AREA( w ) ) )
		return;
	squareSize = (w->allocation.width > w->allocation.height) ?
		w->allocation.height : w->allocation.width;
	glViewport( 0, 0, squareSize, squareSize );
	gtk_gl_area_swapbuffers( GTK_GL_AREA( w ) );
}


void GLWriteName( NetObject* no )
{	int i = 0;
	char* temp;
	temp = IPToString( no->nodeLink->addr );
	glPushMatrix();
	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	glRasterPos3f( no->currentX, no->currentY, no->currentZ );
	while( temp[i] )
	{	glutBitmapCharacter( GLUT_BITMAP_8_BY_13, temp[i] );
		i++;
	}
	glPopMatrix();
}


void GLDrawLink( NetObject* no )
{       GLfloat x1, x2, y1, y2, z1, z2;
	float color, width, newWidth;
	NetObject* me, *other;

	if( ! no || ! no->nodeLink || ! no->netInfoLink || ! no->netInfoLink->nodeLink || ! no->netInfoLink->nodeLink->visual )
		return;
        me = (NetObject*) no->nodeLink->visual;
        other = (NetObject*) no->netInfoLink->nodeLink->visual;
        color = newWidth = no->netInfoLink->totalData;

	color /= 9000.0;
	if( ! gPaused )
	{	NetObjectSetTargetLighting( no, color );
		NetObjectApproachColor( no );
        	newWidth /= 1200.0;
		width = no->lastWidth;
		width += (newWidth-no->lastWidth)/8.0;
		no->lastWidth = width;
	}
	else
	{	width = no->lastWidth;
		if( ! no->initialized )
			return;
	}

	glPushMatrix();
	{	GLfloat lighting[] = { no->currentRed,
			no->currentGreen, no->currentBlue, 1.0 };
		GLfloat position[] = { 0.0, 0.0, 0.0, 1.0 };
		glLightfv( GL_LIGHT0, GL_POSITION, position );
		glLightfv( GL_LIGHT0, GL_AMBIENT, lighting );
	}

/*
** this is the segfault problem. can't find a solution.....
** other is always on 0x10 for the segv, so this is just a check......
** problem seems to be the NetInfo struct loses the nodeLink pointer.
*/
	if( (int) other == 16 )
	{	printf( "**** Error: node %s for node %s is gone", IPToString( no->netInfoLink->addr ), IPToString( no->nodeLink->addr ) );
		return;
	}
        if( width > 10.0 )
        	width = 10.0;
        else if( width < 1.0 )
                width = 1.0;
        x1 = me->currentX;
        y1 = me->currentY;
        z1 = me->currentZ;

        x2 = other->currentX;
        y2 = other->currentY;
        z2 = other->currentZ;

	glColor4f( no->currentRed, no->currentGreen, no->currentBlue, 1.0 );
        glLineWidth( width );
        glBegin( GL_LINE_STRIP );
        glVertex3f( x1, y1, z1 );
        glVertex3f( x2, y2, z2 );
        glEnd();
	glPopMatrix();
}


void GLDrawSphere( NetObject* no )
{	glPushMatrix();
	if( ! no->atCoords && ! gPaused )
		NetObjectApproachCoords( no );
	else if( no->destroyed )
		NetObjectLeaveCoords( no );
	{	GLfloat lighting[] = { no->currentRed,
			no->currentGreen, no->currentBlue, 1.0 };
		GLfloat position[] = { 0.0, 0.0, 0.0, 1.0 };
		glLightfv( GL_LIGHT0, GL_POSITION, position );
		glLightfv( GL_LIGHT0, GL_AMBIENT, lighting );
	}
	glTranslatef( no->currentX, no->currentY, no->currentZ );
	glCallList( GSphere );
	glPopMatrix();
	if( gShowNames == 1 )
		GLWriteName( no );
}


int GLAreaMouseDown( GtkWidget* w, GdkEventButton* eb )
{	static guint32 aTime;
	gMouseDownX = eb->x;
	gMouseDownY = eb->y;
	if( eb->button == 1 && ! gMouseMotion )
		gMouseMotion = gtk_idle_add( (GtkFunction) GLAreaDrag, w );
	if( eb->button == 2 )
	{	gRotateX = 0;
		gRotateY = 0;
	}
	if( eb->button == 3 )
		gtk_menu_popup( GTK_MENU( gMenu ), NULL, NULL, 0, 0, eb->button, aTime++ );
	return 1;
}


int GLAreaMouseUp( GtkWidget* w, GdkEventButton* eb )
{	if( gMouseMotion )
	{	gtk_idle_remove( gMouseMotion );
		gMouseMotion = 0;
	}
	return 1;
}


int GLAreaDrag( GtkWidget* wi, GdkEventMotion* em )
{	gint x, y;
	gint w, h;
	GdkModifierType mod;

	gdk_window_get_pointer( wi->window, &x, &y, &mod );
	w = wi->allocation.width;
	h = wi->allocation.height;
	gRotateX = x - gMouseDownX;
	gRotateX = 12.0 * gRotateX / (float) w;
	if( gRotateX < 0.0 )
		gRotateX = 360.0 + gRotateX;

	gRotateY = y - gMouseDownY;
	gRotateY = 12.0 * gRotateY / (float) h;
	if( gRotateY < 0.0 )
		gRotateY = 360.0 + gRotateY;
	return 1;
}


int GLAreaRedraw( GtkWidget* wi, GdkEvent* event )
{	if( ! gtk_gl_area_make_current( GTK_GL_AREA( wi ) ) )
		return;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix();
	if( ! gPaused )
	{	glRotatef( gRotateX2, 0.0, 1.0, 0.0 );
		glRotatef( gRotateY2, 0.0, 0.0, 1.0 );
	}
	else
	{	glRotatef( 15.0 * gRotateX, 0.0, 1.0, 0.0 );
		glRotatef( 15.0 * gRotateY, 0.0, 0.0, 1.0 );
	}
	if( gNetObjectTree )
		g_tree_traverse( gNetObjectTree, (GTraverseFunc) NetObjectDraw,
			G_IN_ORDER, 0 );

	gRotateY2 += gRotateY;
	if( gRotateY2 > 360.0 ) gRotateY2 -= 360.0;
	else if( gRotateY2 < 0.0 ) gRotateY2 += 360.0;
	gRotateX2 += gRotateX;
	if( gRotateX2 > 360.0 ) gRotateX2 -= 360.0;
	else if( gRotateX2 < 0.0 ) gRotateX2 += 360.0;
	glPopMatrix();

	gtk_gl_area_swapbuffers( GTK_GL_AREA( wi ) );
	return 1;
}
