#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkgl/gdkgl.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <gtkgl/gtkglarea.h>

#include "gtkdisplay.h"
#include "gldisplay.h"
#include "wired.h"


extern int gGLUpdateTime;


GtkWidget* GMainWindow;
GtkWidget* GGLArea;
GtkWidget* gMenu;

int GAttribList[] = {GDK_GL_RGBA, GDK_GL_RED_SIZE, 1, GDK_GL_GREEN_SIZE, 1,
	GDK_GL_BLUE_SIZE, 1, GDK_GL_DEPTH_SIZE, 1, GDK_GL_DOUBLEBUFFER,
	GDK_GL_NONE };



void DestroyMainWindow()
{	WiredInterfaceDestroyAll();
	gtk_main_quit();
}


int DestroyMe( GtkWidget* notme, gpointer me )
{	gtk_widget_destroy( GTK_WIDGET( me ) );
}


void About( GtkMenuItem* menuItem )
{	GtkWidget* about;
	GtkWidget* ok;
	GtkWidget* label;

	static char* aboutString = "WiredView 0.0.1\njohn white <jkwhite@ucsd.edu>\n ";

	about = gtk_dialog_new();
	gtk_container_border_width( GTK_CONTAINER(about), 10 );

	label = gtk_label_new( aboutString );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( about )->vbox ), label,
		TRUE, TRUE, 0 );
	gtk_widget_show( label );

	ok = gtk_button_new_with_label( "OK" );
	gtk_signal_connect( GTK_OBJECT( ok ), "clicked",
		GTK_SIGNAL_FUNC( DestroyMe ), about );
	gtk_box_pack_start( GTK_BOX( GTK_DIALOG( about )->vbox),
		ok, TRUE, TRUE, 0 );
	GTK_WIDGET_SET_FLAGS( ok, GTK_CAN_DEFAULT );
	gtk_widget_grab_default( ok );
	gtk_widget_show( ok );
	gtk_widget_show( about );
}


void SetUpMenu()
{	GtkWidget* menuItem;
	GtkWidget* optionsMenu;
	GtkWidget* about;

	optionsMenu = gtk_menu_new();
	menuItem = gtk_menu_item_new_with_label( "Show Addresses" );
	gtk_menu_item_configure( GTK_MENU_ITEM( menuItem ), 1, 0 );
	gtk_signal_connect( GTK_OBJECT( menuItem ), "activate",
		(GtkSignalFunc) GLShowAddresses, 0 );
	gtk_menu_append( GTK_MENU( optionsMenu ), menuItem );
	gtk_widget_show( optionsMenu );
	gtk_widget_show( menuItem );

	gMenu = gtk_menu_new();
	menuItem = gtk_menu_item_new_with_label( "Pause/Unpause" );
	gtk_signal_connect( GTK_OBJECT( menuItem ), "activate",
		(GtkSignalFunc) GLDisplayPause, 0 );
	gtk_menu_append( GTK_MENU( gMenu ), menuItem );
	gtk_widget_show( menuItem );
	menuItem = gtk_menu_item_new_with_label( "Options" );
	gtk_menu_item_configure( GTK_MENU_ITEM( menuItem ), 0, 1 );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( menuItem ), optionsMenu );
	gtk_menu_append( GTK_MENU( gMenu ), menuItem );
	gtk_widget_show( menuItem );

	menuItem = gtk_menu_item_new_with_label( "About" );
	gtk_signal_connect( GTK_OBJECT( menuItem ), "activate",
		(GtkSignalFunc) About, 0 );
	gtk_menu_append( GTK_MENU( gMenu ), menuItem );
	gtk_widget_show( menuItem );

	gtk_widget_show( gMenu );
}


void SetUpGLArea( GtkWidget* glArea )
{	//gtk_widget_set_usize( GTK_WIDGET( glArea ), 400, 400 );
	gtk_widget_set_events( glArea, GDK_EXPOSURE_MASK
		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );
	gtk_signal_connect( GTK_OBJECT( glArea ), "expose_event",
		(GtkSignalFunc) GLAreaRedraw, (gpointer) 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "realize",
		(GtkSignalFunc) GLAreaInit, 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "show",
		(GtkSignalFunc) GLAreaRedraw, 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "size_request",
		(GtkSignalFunc) GLAreaResize, 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "motion_notify_event",
		(GtkSignalFunc) GLAreaDrag, 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "button_release_event",
		(GtkSignalFunc) GLAreaMouseUp, 0 );
	gtk_signal_connect( GTK_OBJECT( glArea ), "button_press_event",
		(GtkSignalFunc) GLAreaMouseDown, 0 );
	g_timeout_add( gGLUpdateTime, (GSourceFunc) GLAreaRedraw, glArea );
/* can't find any other way around automatic window sizing */
	g_timeout_add( 500, (GSourceFunc) GLAreaInitResize, glArea );
}


int InitGtk( int argc, char** argv )
{	gtk_init( &argc, &argv );

	GMainWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title( GTK_WINDOW( GMainWindow ), "WiredView" );
	gtk_signal_connect( GTK_OBJECT( GMainWindow ), "destroy",
		GTK_SIGNAL_FUNC( DestroyMainWindow ), NULL );

	GGLArea = gtk_gl_area_new( GAttribList );
	if( ! GGLArea )
	{	printf( "Could not acquire necessary visual!\n" );
		exit(1);
	}
	SetUpMenu();
	SetUpGLArea( GGLArea );

	gtk_container_add( GTK_CONTAINER( GMainWindow ), GGLArea );
	gtk_widget_show( GGLArea );
	gtk_widget_show( GMainWindow );
	gtk_main();
	return 0;
}
