#include <stdlib.h>
#include <math.h>
#include <glib.h>

#include "config.h"
#include "netobject.h"
#include "gldisplay.h"
#include "wired.h"
#include "support.h"
#include "GL/gl.h"


GTree* gNetObjectTree;

extern float gRotationRadius;
extern int gPaused;


gpointer AddNodeVisual( Node* n )
{       NetObject* new = AddNetObject( NetObjectCreate( GLDrawSphere ) );
	new->type = NO_NODE;
        new->nodeLink = n;
	new->netInfoLink = 0;
	if( ! gNetObjectTree )
		gNetObjectTree = g_tree_new( (GCompareFunc) KeyCompare );
	g_tree_insert( gNetObjectTree, (gpointer) IPToKey( n->addr ), new );
        return new;
}


void RemoveNodeVisual( Node* n )
{	NetObject* no;
	no = g_tree_lookup( gNetObjectTree, (gpointer) IPToKey( n->addr ) );
	g_tree_remove( gNetObjectTree, (gpointer) IPToKey( n->addr ) );
	DEBUG(1) printf( "Removing node visual %s\n", IPToString( n->addr ) );
	n->visual = 0;
	NetObjectDestroy( no );
}


gpointer AddLinkVisual( NetInfo* ni )
{	NetObject* new = AddNetObject( NetObjectCreate( GLDrawLink ) );
	Node* n = ni->myself;
	NetObject* node = (NetObject*) ni->myself->visual;
	GTree* tree;
	DEBUG(1) printf( "Adding link from %s to %s\n", IPToString( n->addr ),
		IPToString( ni->nodeLink->addr ) ); 
	new->type = NO_LINK;
	new->nodeLink = n;
	new->netInfoLink = ni;
	if( ni->isSent )
	{	if( ! node->sentLinkTree )
			node->sentLinkTree = g_tree_new( KeyCompare );
		tree = node->sentLinkTree;
	}
	else
	{	if( ! node->receivedLinkTree )
			node->receivedLinkTree = g_tree_new( KeyCompare );
		tree = node->receivedLinkTree;
	}
	g_tree_insert( tree, (gpointer) IPToKey( ni->addr ), new );
	ni->visual = new;
        return new;
}


void RemoveLinkVisual( NetInfo* ni )
{	Node* n = ni->myself;
        NetObject* no = (NetObject*) n->visual;
	NetObject* link = (NetObject*) ni->visual;
	GTree* tree;
	gint n1, n2;
	DEBUG(1) printf( "Removing visual link from %s to %s\n", IPToString( n->addr ),
		IPToString( ni->addr ) ); 
	if( ni->isSent )
		tree = no->sentLinkTree;
	else
		tree = no->receivedLinkTree;

	n1 = g_tree_nnodes( tree );
        g_tree_remove( tree, (gpointer) IPToKey( ni->addr ) );
	n2 = g_tree_nnodes( tree );
	if( n2 != n1 - 1 )
		DEBUG(1) printf( "Could not find node %s in %s link tree of %s\n",
		IPToString( ni->addr ), (ni->isSent)?"sent":"rec", IPToString( n->addr ) );
        NetObjectDestroy( link );
	ni->visual = 0;
}


void StopDestroyingVisual( gpointer visual )
{	NetObject* no = (NetObject*) visual;
	if( no->type == NO_NODE && no->destroyed )
 	{	no->atCoords = 0;
                no->destroyed = 0;
	}
}


void BeginDestroyVisual( gpointer visual )
{	NetObjectSetDestroyed( (NetObject*) visual );
}


int DestroyingVisual( gpointer visual )
{	return ((NetObject*)visual)->destroyed;
}


gint NetCompare( NetObject* n1, NetObject* n2 )
{	gint32 k;

	k = IPToKey( n1->nodeLink->addr ) - IPToKey( n2->nodeLink->addr );
	if( k < 0 )
		k = -1;
	else if( k > 0 )
		k = 1;
	return k;
}


gint NetLinkCompare( NetObject* n1, NetObject* n2 )
{	gint32 k;

	k = IPToKey( n1->netInfoLink->addr ) - IPToKey( n2->netInfoLink->addr );
	if( k < 0 )
		k = -1;
	else if( k > 0 )
		k = 1;
	return k;
}


NetObject* NetObjectCreate( void (*draw)( NetObject* ) )
{       NetObject* no;

        no = (NetObject*) malloc( sizeof( NetObject ) );
        no->DrawSelf = draw;
        no->targetRed = 0;
        no->targetGreen = 0;
        no->targetBlue = 0;
        no->currentRed = 0;
        no->currentGreen = 0;
        no->currentBlue = 0;
        no->destroyed = 0;
        no->initialized = 0;
	no->sentLinkTree = 0;
	no->receivedLinkTree = 0;
	no->lastWidth = 0;
        return no;
}


void NetObjectDestroy( NetObject* no )
{	if( no->sentLinkTree )
		free( no->sentLinkTree );
	if( no->receivedLinkTree )
		free( no->receivedLinkTree );
        free( no );
}


void NetObjectSetTargetColor( NetObject* no, float r, float g, float b )
{       no->targetRed = r;
        no->targetGreen = g;
        no->targetBlue = b;
}


void NetObjectSetTargetLighting( NetObject* no, float amount )
{       no->targetRed = 3.0 * amount - 0.6;
        no->targetGreen = 2.0 * (0.5 - fabs(-amount+0.5) - 0.0);
        no->targetBlue = 1.0 - 1.5 * amount;
        if( no->targetRed < 0 )
                no->targetRed = 80.0 * (0.05 - fabs(-amount+0.05));
        if( no->targetGreen < 0 )
                no->targetGreen = 2.0 * amount - 2.0;
        if( no->targetGreen < 0 )
                no->targetGreen = 0;
        if( no->targetBlue < 0 )
                no->targetBlue = 0;
}


void NetObjectApproachColor( NetObject* no )
{       no->currentRed += (no->targetRed - no->currentRed)/4.0;
        no->currentGreen += (no->targetGreen - no->currentGreen)/4.0;
        no->currentBlue += (no->targetBlue - no->currentBlue)/4.0;
}


int NetObjectLeaveCoords( NetObject* no )
{       no->currentX *= 1.2;
        no->currentY *= 1.2;
        no->currentZ *= 1.2;
        return 0;
}


int NetObjectApproachCoords( NetObject* no )
{       no->currentX += (no->targetX - no->currentX)/2.0;
        no->currentY += (no->targetY - no->currentY)/2.0;
        no->currentZ += (no->targetZ - no->currentZ)/2.0;
        if( no->currentX == no->targetX && no->currentY == no->targetY
                && no->currentZ == no->targetZ )
        {       no->atCoords = 1;
        }
        return 0;
}


void NetObjectSetDestroyed( NetObject* no )
{       no->destroyed = 1;
        no->atCoords = 1;       /* in case of bad floating-point compares */
}


NetObject* AddNetObject( NetObject* no )
{       float ry;

        /* -R <= x <= R */
        no->targetX = 2.0 * gRotationRadius * (float) drand48() - gRotationRadius;
        /* Ry = sqrt( R^2 - x^2 ) */
        ry = sqrt( gRotationRadius * gRotationRadius - no->targetX * no->targetX );
        /* -Ry <= y <= Ry */
        no->targetY = (float) 2.0 * ry * drand48() - ry;
        /* z is the value which puts (x,y,z) on the surface of the sphere */
        no->targetZ = sqrt( ry * ry - no->targetY * no->targetY );
        if( rand() % 2 )
                no->targetZ = -no->targetZ;     /* or t'other way 'round */
        no->currentX = 6.0 * no->targetX;
        no->currentY = 6.0 * no->targetY;
        no->currentZ = 6.0 * no->targetZ;
        no->atCoords = 0;
        return no;
}


int NetObjectDrawLink( gpointer unused, NetObject* no )
{	no->DrawSelf( no );
	return 0;
}


int NetObjectDraw( gpointer unused1, NetObject* no, gpointer unused2 )
{	float color;
        if( ! no )
                return 0;
        color = no->nodeLink->nodeStats->totalData;
        color /= 9000.0;
	if( ! gPaused )
        {	NetObjectSetTargetLighting( no, color );
      		NetObjectApproachColor( no );
	}
        no->DrawSelf( no );
	if( no->sentLinkTree )
		g_tree_traverse( no->sentLinkTree,
			(GTraverseFunc) NetObjectDrawLink, G_IN_ORDER, 0 );
	if( no->receivedLinkTree )
		g_tree_traverse( no->receivedLinkTree,
			(GTraverseFunc) NetObjectDrawLink, G_IN_ORDER, 0 );
        return 0;
}


int NetObjectInitialize( gpointer unused1, NetObject* no )
{	no->initialized = 1;
	return 1;
}


void NetObjectInitializeAll()
{	if( gNetObjectTree )
		g_tree_traverse( gNetObjectTree,
			(GTraverseFunc) NetObjectInitialize, G_IN_ORDER, 0 );
}
