#include <glib.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>

#include "config.h"
#include "node.h"
#include "wired.h"
#include "netobject.h"
#include "netstat.h"
#include "support.h"


extern int gPaused;
extern float gNodeTimeoutTime;

GTree* GNodeTree;

GSList* gDestroyedList;


int PrintAllNodes()
{	if( ! GNodeTree )
		return;
	g_tree_traverse( GNodeTree, (GTraverseFunc) NodeDisplayNodeInfo,
		G_IN_ORDER, 0 );
	return 1;
}


Node* CreateNode( ip* addr )
{	Node* n = (Node*) malloc( sizeof( Node ) );
	n->nodeStats = NetStatCreate( addr );
	memcpy( n->addr, addr, 4 * sizeof( ip ) );
	n->visual = AddNodeVisual( n );
	return n;
}


void NodeDestroy( Node* n )
{	char temp[20];
	NetStat* ns;
	n->nodeStats = 0;
//	NetStatDestroy( ns );
	free( n );
}


int NodeForget( gpointer unused, Node* n, Node* f )
{	NetStatForget( n->nodeStats, f );
	return 0;
}


Node* FindNode( ip* addr )
{	if( ! GNodeTree )
		GNodeTree = g_tree_new( &KeyCompare );
	return g_tree_lookup( GNodeTree, (gpointer) IPToKey( addr ) );
}


Node* AddNewIP( ip* addr )
{	char temp[20];
	Node* n = CreateNode( addr );
	DEBUG(1) printf( "Adding node %s\n", IPToString( addr ) );
	AddNode( n );
	return n;
}


int AddNode( Node* new )
{	if( ! GNodeTree )
		GNodeTree = g_tree_new( &KeyCompare );

	g_tree_insert( GNodeTree, (gpointer) NodeKey( new ), new );
}


int NodeKey( Node* n )
{	return IPToKey( n->addr );
}


int NodeProcessData( Node* n, ip* source, ip* dest, struct timeval* timestamp,
	guint32 data )
{	if( ! IPCompare( n->addr, source ) )
		return NetStatProcessSend( n->nodeStats, n, dest, timestamp, data );
	else
		return NetStatProcessReceive( n->nodeStats, n, source, timestamp, data);
}


void NodeUpdate( WiredInterface* wi, struct timeval* current )
{	if( GNodeTree )
		g_tree_traverse( GNodeTree, (GTraverseFunc) &NodeUpdateNode, G_IN_ORDER, (gpointer) current );
	if( gDestroyedList )
		NodeProcessDestroyed();
}


void NodeDisplayInfo( WiredInterface* wi )
{	if( GNodeTree )
		g_tree_traverse( GNodeTree, &NodeDisplayNodeInfo, G_IN_ORDER, 0 );
}


int NodeDisplayNodeInfo( gpointer key, gpointer g, gpointer unused2 )
{	char temp[20];
	Node* n = (Node*) g;

	printf( "Address %s has key %lu\n", IPToString( n->addr ), (guint32) key);
	
//	NetStatDisplayInfo( n->nodeStats );
	return 0;
}


int NodeUpdateNode( gpointer unused1, Node* n, struct timeval* current )
{	float t = GetTimeDifference2( &n->nodeStats->lastPacket, current );

	if( ! gPaused )
	{	if( t > gNodeTimeoutTime && ! DestroyingVisual( n->visual ) )
		{	DEBUG(1) printf( "Node %s time is %f--destroying\n", IPToString(n->addr), t );
			BeginDestroyVisual( n->visual );
		}
		else if( t > gNodeTimeoutTime + 1000.0 )
			NodeQueueDestroy( n );
		else if( t < gNodeTimeoutTime && DestroyingVisual( n->visual ) )
			StopDestroyingVisual( n->visual );
	}
	NetStatUpdate( n->nodeStats, current );
	return 0;
}


void NodeQueueDestroy( Node* n )
{	if( ! gDestroyedList )
		gDestroyedList = g_slist_alloc();
	g_slist_append( gDestroyedList, n );
}


void NodeProcessDestroyed()
{	Node* d;
        int i = g_slist_length( gDestroyedList );
        int k;

        while( i-- > 1 )
        {       d = g_slist_nth_data( gDestroyedList, i );
		RemoveNodeVisual( d );
                k = IPToKey( d->addr );
                g_tree_remove( GNodeTree, (gpointer) IPToKey( d->addr ) );
                DEBUG(1) printf( "Total nodes now: %d\n", g_tree_nnodes( GNodeTree ) );
                g_tree_traverse( GNodeTree, (GTraverseFunc) NodeForget, G_IN_ORDER, d );
                NodeDestroy( d );
        }
        g_slist_free( gDestroyedList );
        gDestroyedList = 0;
}
