#include <glib.h>
#include <pcap/pcap.h>
#include <stdlib.h>
#include <gdk/gdk.h>
#include <sys/time.h>
#include <unistd.h>

#include "wired.h"
#include "node.h"


GSList* GWiredInterfaceList;

guint32 GGlobalData;
guint32 GGlobalDataSwap;



/*	--use the other IPToString instead
char* IPToString( ip* addr )
{	static char buf[16];
	sprintf( buf, "%d.%d.%d.%d", addr[0], addr[1], addr[2], addr[3] );
	return buf;
}
*/


/*	--this function is straight from ethereal; it's faster
**	than the above function. sorry for the reformatting :)
*/
gchar* IPToString( const ip* addr )
{	static gchar str[3][16];
	static gchar *cur;
	gchar *p;
	int i;
	guint32 octet;
	guint32 digit;

	if (cur == &str[0][0])
	{	cur = &str[1][0];
	}
	else if (cur == &str[1][0])
	{	cur = &str[2][0];
	}
	else
	{	cur = &str[0][0];
	}
	p = &cur[16];
	*--p = '\0';
	i = 3;
	for (;;)
	{	octet = addr[i];
		*--p = (octet % 10) + '0';
		octet /= 10;
		digit = octet % 10;
		octet /= 10;
		if (digit != 0 || octet != 0)
			*--p = digit + '0';
		if (octet != 0)
			*--p = octet + '0';
		if (i == 0)
			break;
		*--p = '.';
		i--;
	}
	return p;
}


gint IPCompare( const ip* n1, const ip* n2 )
{       return IPToKey( n1 ) - IPToKey( n2 );
}


gint KeyCompare( gconstpointer k1, gconstpointer k2 )
{	if( ((guint32) k1) > ((guint32) k2) )
		return 1;
	else if( ((guint32) k1) < ((guint32) k2) )
		return -1;
	else
		return 0;
}


guint32 IPToKey( const ip* addr )
{	guint32 k = *((guint32*)addr);
	return k;
}


guint32 WiredInterfaceGetGlobalTraffic()
{	printf( "%lu\n", GGlobalData );
	return GGlobalData;
}


void WiredInterfaceUpdateAll()
{	GGlobalData = GGlobalDataSwap;
	GGlobalDataSwap = 0;
	WiredInterfaceApply( &WiredInterfaceUpdate );
}


void WiredInterfaceUpdate( WiredInterface* wi )
{	struct timeval current;

	gettimeofday( &current, 0 );
	wi->data = 0;
	NodeUpdate( wi, &current );
}


int WiredInterfaceDisplayInfoAll()
{	WiredInterfaceApply( WiredInterfaceDisplayInfo );
	return 1;
}


void WiredInterfaceFree( WiredInterface* wi )
{	free( wi );
}


void WiredInterfaceDestroyAll()
{	WiredInterfaceApply( WiredInterfaceClose );
	WiredInterfaceApply( WiredInterfaceFree );
	g_slist_free( GWiredInterfaceList );
	GWiredInterfaceList = 0;
}


int WiredInterfaceEnable( WiredInterface* wi )
{	if( wi )
	{	wi->enabled = 1;
		gdk_input_add( wi->pcapFd, GDK_INPUT_READ,
			(GdkInputFunction) WiredInterfaceProcess,
			wi );
	}
	return 0;
}


int WiredInterfaceDisable( WiredInterface* wi )
{	wi->enabled = 0;
}


WiredInterface* WiredInterfaceCreate( char* device )
{	WiredInterface* wi;

	wi = (WiredInterface*) malloc( sizeof( WiredInterface ) );
	wi->enabled = 0;
	wi->p = 0;
	wi->device = device;
	if( ! GWiredInterfaceList )
		GWiredInterfaceList = g_slist_alloc();
	g_slist_append( GWiredInterfaceList, wi );
	if( WiredInterfaceInit( wi ) )
	{	WiredInterfaceDestroy( wi );
		return 0;
	}
	return wi;
}


WiredInterface* WiredInterfaceCreateEnabled( char* device )
{	WiredInterface* wi = WiredInterfaceCreate( device );
	if( wi )
		WiredInterfaceEnable( wi );
	return wi;
}


void WiredInterfaceDestroy( WiredInterface* wi )
{	g_slist_remove( GWiredInterfaceList, wi );
	WiredInterfaceClose( wi );
	free( wi );
}


void WiredInterfaceDisplayInfo( WiredInterface* wi )
{	printf( "Device %s\n", wi->device );
	NodeDisplayInfo( wi );
}


int WiredInterfaceProcess( WiredInterface* wi, gint s,
	GdkInputCondition condition )
{	u_char* packetData;
	struct timeval timestamp;
	ip* packet;
	ip* source;
	ip* dest;
	Node* sourceN, *destN;
	guint32 data;

	if( !(packet = (ip*) pcap_next( wi->p, &wi->header )) )
		return 0;
	source = packet + 26;
	dest = packet + 30;
	timestamp.tv_sec = wi->header.ts.tv_sec;
	timestamp.tv_usec = wi->header.ts.tv_usec;
	data = wi->header.len;
	wi->data += data;
	GGlobalDataSwap += data;
	if( ! (sourceN = FindNode( source ) ) )
		sourceN = AddNewIP( source );
	if( ! (destN = FindNode( dest ) ) )
		destN = AddNewIP( dest );
	NodeProcessData( sourceN, source, dest, &timestamp, data );
	NodeProcessData( destN, source, dest, &timestamp, data );
	return 0;
}


int WiredInterfaceInit( WiredInterface* wi )
{	char error[PCAP_ERRBUF_SIZE];

	if( ! wi->device )
	{	if( ! (wi->device = pcap_lookupdev( error ) ) )
		{	printf( "Wired: Error: %s\n", error );
			return 1;
		}
	}
	if( ! (wi->p = pcap_open_live( wi->device, MAXSIZE, TRUE, 5, error ) ) )
	{	printf( "Wired: Error: %s\n", error );
		return 1;
	}
	wi->pcapFd = pcap_fileno( wi->p );
	return 0;
}


void WiredInterfaceClose( WiredInterface* wi )
{	if( wi && wi->p )
		pcap_close( wi->p );
}


void WiredInterfaceApply( void (*function)(WiredInterface*) )
{	int i = g_slist_length( GWiredInterfaceList );

	while( i-- > 1 )
	{	function( (WiredInterface*)
			g_slist_nth_data( GWiredInterfaceList, i ) );
	}
}
