#ifndef WIRED_H
#define WIRED_H


#include <glib.h>
#include <pcap/pcap.h>
#include <gdk/gdk.h>


#define MAXSIZE 120


typedef guint8 ip;

typedef struct wired_interface
{	char* device;
	pcap_t* p;
	struct pcap_pkthdr header;
	int pcapFd;
	int enabled;
	guint32 data;
} WiredInterface;


/* public */

/* compare and misc functions */
gchar* IPToString( const ip* addr );
gint IPCompare( const ip* n1, const ip* n2 );
gint KeyCompare( gconstpointer k1, gconstpointer k2 );
guint32 IPToKey( const ip* addr );


guint32 WiredInterfaceGetGlobalTraffic();
void WiredInterfaceUpdateAll();
void WiredInterfaceUpdate( WiredInterface* wi );
int WiredInterfaceDisplayInfoAll();
void WiredInterfaceDestroyAll();

WiredInterface* WiredInterfaceCreate( char* device );
WiredInterface* WiredInterfaceCreateEnabled( char* device );
void WiredInterfaceDestroy( WiredInterface* wi );
int WiredInterfaceEnable( WiredInterface* wi );
int WiredInterfaceDisable( WiredInterface* wi );
void WiredInterfaceDisplayInfo( WiredInterface* wi );
void WiredInterfaceApply( void (*Func)(WiredInterface*) );


/* private */

int WiredInterfaceInit( WiredInterface* wi );
int WiredInterfaceProcess( WiredInterface* wi, gint, GdkInputCondition );
void WiredInterfaceClose( WiredInterface* wi );
void WiredInterfaceFree( WiredInterface* wi );


#endif
